#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pygame
import pygame.locals as pg


LEVEL_MAP = [
    "XXXXXXXXXXXXXXXXXXXXXXXXX",
    "X.......................X",
    "X.XXXXXXXXXX.XXXXXXXXXX.X",
    "X.X......X.X..........X.X",
    "X.X..X.XXX.X.XXXXXXXXXX.X",
    "X.XXXX.....X.......X..X.X",
    "X......XXXXXXXXXXX.X..X.X",
    "X.XX.X.X.........X....X.X",
    "X.X..X...........X.X..X.X",
    "X.X..X.X.........X.X..X.X",
    "X.X..X.XXXXXXXXXXX.XXXX.X",
    "X.XXXX..................X",
    "X.X..X.XXXXXXXXXXXXXXXX.X",
    "X.X..X................X.X",
    "X.X....X..............X.X",
    "X.XXXX.XXXXXXXXXXXXXXXX.X",
    "X.......................X",
    "XXXXXXXXXXXXXXXXXXXXXXXXX",
]

def initialize_screen():
    pygame.display.init()
    screen = pygame.display.set_mode((800, 600))
    screen.fill((0x66, 0x66, 0x66))
    return screen

def load_images():
    images = {
        'X': pygame.image.load('wall.png').convert(),
        '.': pygame.image.load('dirt.png').convert(),
        '@': pygame.image.load('hero.png').convert(),
    }
    return images

def draw_map(screen, images, level_map):
    for y, row in enumerate(level_map):
        for x, square in enumerate(row):
            screen.blit(images[square], (x * 32, y * 32))

def wait_for_quit(screen, images, level_map):
    hero_x, hero_y = 1, 1
    level_map = [list(row) for row in level_map]
    level_map[hero_y][hero_x] = '@'
    while True:
        draw_map(screen, images, level_map)
        pygame.display.update()
        event = pygame.event.wait()
        if event.type == pg.QUIT:
            return

def run_game():
    screen = initialize_screen()
    images = load_images()
    wait_for_quit(screen, images, LEVEL_MAP)


if __name__ == '__main__':
    run_game()
