#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pygame
import pygame.locals as pg


def load_sheet(filename, width=32, height=None):
    if height is None:
        height = width
    image = pygame.image.load(filename)
    image = image.convert()
    image.set_colorkey((0xff, 0, 0xff))
    w, h = image.get_size()
    images = []
    for x in xrange(w // width):
        for y in xrange(h // height):
            rect = pygame.rect.Rect(x * width, y * height, width, height)
            images.append(image.subsurface(rect))
    return images


class Game(object):
    def __init__(self):
        pygame.display.init()
        pygame.font.init()
        self.screen = pygame.display.set_mode((800, 600))
        self.screen.fill((0x66, 0x66, 0x66))
        self.tilesheet = load_sheet('tilesheet.png')
        self.font = pygame.font.Font(None, 16)

    def display(self):
        for i, tile in enumerate(self.tilesheet):
            x, y = divmod(i, 600 // 33)
            self.screen.blit(tile, (x * 33 + 1, y * 33 + 1))
            label = self.font.render('%d' % i, False, (0xff, 0xff, 0xff))
            back = self.font.render('%d' % i, False, (0x00, 0x00, 0x00))
            self.screen.blit(back, (x * 33 + 1 - 1, y * 33 + 1))
            self.screen.blit(back, (x * 33 + 1 + 1, y * 33 + 1))
            self.screen.blit(back, (x * 33 + 1, y * 33 + 1 - 1))
            self.screen.blit(back, (x * 33 + 1, y * 33 + 1 + 1))
            self.screen.blit(label, (x * 33 + 1, y * 33 + 1))
        pygame.display.update()

    def control(self):
        event = pygame.event.wait()
        if event.type == pg.QUIT:
            self.game_over = True

    def run(self):
        self.game_over = False
        while not self.game_over:
            self.display()
            self.control()

if __name__ == '__main__':
    game = Game()
    game.run()
