#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pygame
import pygame.locals as pg
from pytmx import tmxloader


class Level(object):
    def __init__(self, filename):
        self.tiled_map = tmxloader.load_pygame(filename)

    def render(self, screen):
        w = self.tiled_map.tilewidth
        h = self.tiled_map.tileheight
        get_image = self.tiled_map.getTileImage
        for l, layer in enumerate(self.tiled_map.tilelayers):
            for y in xrange(self.tiled_map.height):
                for x in xrange(self.tiled_map.width):
                    image = get_image(x, y, l)
                    if image:
                        screen.blit(image, (x * w, y * h))


class Game(object):
    def __init__(self):
        pygame.display.init()
        pygame.font.init()
        self.screen = pygame.display.set_mode((800, 600))
        self.screen.fill((0x66, 0x66, 0x66))

    def load_level(self, filename):
        self.level = Level(filename)

    def display(self):
        self.level.render(self.screen)
        pygame.display.update()

    def control(self):
        event = pygame.event.wait()
        if event.type == pg.QUIT:
            self.game_over = True

    def run(self):
        self.game_over = False
        while not self.game_over:
            self.display()
            self.control()


if __name__ == '__main__':
    game = Game()
    game.load_level('level.tmx')
    game.run()
