# -*- coding: utf-8 -*-

import pygame


FACING = {
    (0, -1): 0,
    (-1, 0): 1,
    (0, 1): 2,
    (1, 0): 3,
}


def load_sheet(filename, width=32, height=None):
    if height is None:
        height = width
    image = pygame.image.load(filename)
    image = image.convert()
    image.set_colorkey((0xff, 0, 0xff))
    w, h = image.get_size()
    images = []
    for x in xrange(w // width):
        for y in xrange(h // height):
            rect = pygame.rect.Rect(x * width, y * height, width, height)
            images.append(image.subsurface(rect))
    return images


class ShadowSprite(pygame.sprite.Sprite):
    def __init__(self, sprite):
        super(ShadowSprite, self).__init__()
        self.spritesheet = load_sheet('effects.png', 32)
        self.image = self.spritesheet[0]
        self.rect = self.image.get_rect()
        self.image.set_alpha(0.6 * 255)
        self.sprite = sprite
        self.rect.midbottom = self.sprite.rect.midbottom

    def update(self):
        self.rect.midbottom = self.sprite.rect.midbottom


class HeroSprite(pygame.sprite.Sprite):
    def __init__(self, (x, y)):
        super(HeroSprite, self).__init__()
        self.spritesheet = load_sheet('spritesheet.png', 64)
        self.facing = 2
        self.image = self.spritesheet[self.facing]
        self.rect = self.image.get_rect()
        self.rect.midbottom = (x * 32 + 16, y * 32 + 32)
        self.animations = []

    def move(self, (dx, dy)):
        self.facing = FACING[dx, dy]
        self.animations.append(self.animation_walk((dx, dy)))

    def animation_walk(self, (dx, dy)):
        for step in xrange(8):
            self.image = self.spritesheet[self.facing + step * 4 + 4]
            self.rect.move_ip(dx * 4, dy * 4)
            yield

    def animation_stand(self):
        self.image = self.spritesheet[self.facing]
        return
        yield

    def update(self):
        if not self.animations:
            self.animations.append(self.animation_stand())
        while self.animations:
            try:
                self.animations[0].next()
                return
            except StopIteration:
                self.animations.pop(0)
