# -*- coding: utf-8 -*-

import pygame
import pygame.locals as pg

import level
import sprites


class Game(object):
    def __init__(self):
        self.screen = pygame.display.get_surface()
        self.screen.fill((0x66, 0x66, 0x66))
        self.background = self.screen.copy()
        self.keys = {
            pg.K_UP: (self.move_hero, (0, -1)),
            pg.K_DOWN: (self.move_hero, (0, 1)),
            pg.K_LEFT: (self.move_hero, (-1, 0)),
            pg.K_RIGHT: (self.move_hero, (1, 0)),
            pg.K_ESCAPE: (self.quit,),
        }
        self.shadow_sprites = pygame.sprite.RenderUpdates()
        self.sprites = sprites.SortedUpdates()
        self.key = None

    def load_level(self, filename):
        self.level = level.Level(filename)
        for name, pos in self.level.objects():
            sprite = sprites.DecorationSprite(pos)
            self.sprites.add(sprites.DecorationSprite(pos))
            self.shadow_sprites.add(sprites.ShadowSprite(sprite))
        self.hero_sprite = sprites.HeroSprite(self.level.hero_position)
        self.sprites.add(self.hero_sprite)
        self.shadow_sprites.add(sprites.ShadowSprite(self.hero_sprite))

    def refresh(self):
        self.level.render(self.background)
        self.screen.blit(self.background, (0, 0))
        self.sprites.draw(self.screen)
        pygame.display.update()

    def quit(self):
        self.game_over = True

    def move_hero(self, direction):
        if self.level.move_hero(direction):
            self.hero_sprite.move(direction)

    def control(self):
        for event in pygame.event.get():
            if event.type == pg.QUIT:
                self.quit()
            elif event.type == pg.KEYDOWN:
                self.key = event.key
        if not self.hero_sprite.animations:
            keys = pygame.key.get_pressed()
            for key, action in self.keys.iteritems():
                if keys[key] or self.key == key:
                    action[0](*action[1:])
                    break
            self.key = None

    def display(self):
        self.sprites.clear(self.screen, self.background)
        self.shadow_sprites.clear(self.screen, self.background)
        self.sprites.update()
        self.shadow_sprites.update()
        dirty = []
        dirty.extend(self.shadow_sprites.draw(self.screen))
        dirty.extend(self.sprites.draw(self.screen))
        pygame.display.update(dirty)

    def run(self, scenes):
        self.refresh()
        self.clock = pygame.time.Clock()
        self.game_over = False
        while not self.game_over:
            self.display()
            self.control()
            self.clock.tick(30)

