# -*- coding: utf-8 -*-

from pytmx import tmxloader

import resources


class Level(object):
    def __init__(self, filename):
        path = resources.file_path(filename)
        self.tiled_map = tmxloader.load_pygame(path)
        self.hero_position = 1, 1

    def render(self, screen):
        w = self.tiled_map.tilewidth
        h = self.tiled_map.tileheight
        get_image = self.tiled_map.getTileImage
        for l, layer in enumerate(self.tiled_map.tilelayers):
            for y in xrange(self.tiled_map.height):
                for x in xrange(self.tiled_map.width):
                    image = get_image(x, y, l)
                    if image:
                        screen.blit(image, (x * w, y * h))
        x, y = self.hero_position

    def get_properties(self, (x, y)):
        properties = {}
        for l, layer in enumerate(self.tiled_map.tilelayers):
            properties.update(self.tiled_map.getTileProperties((x, y, l)) or {})
        return properties

    def is_blocked(self, (x, y)):
        return self.get_properties((x, y)).get('block')

    def move_hero(self, (dx, dy)):
        x, y = self.hero_position
        if not self.is_blocked((x + dx, y + dy)):
            self.hero_position = x + dx, y + dy
            return True
        return False

    def objects(self):
        w = self.tiled_map.tilewidth
        h = self.tiled_map.tileheight
        for og in self.tiled_map.objectgroups:
            for o in og:
                yield o.name, (o.x // w, o.y // h)

